//+--------------------------------------------------------------------------------+
//|                                        FIFO_CloseAll_Options.mq4               |
//|                                      Copyright 2017, nicholishen               |
//|                         https://www.forexfactory.com/nicholishen               |
//|  https://www.forexfactory.com/thread/716973-maximum-overkill-close-all-script  |
//+--------------------------------------------------------------------------------+

/* VERSION HISTORY:
v1.00  2017 - by nicholishen of ForexFactory.com
   Posted in:  https://www.forexfactory.com/thread/716973-maximum-overkill-close-all-script

v1.01  2021-Feb-14 - by pips4life
   Removed the 2nd sort, "list.Sort(0)", because it's definitely wrong to sort last by ticket#.  
      For the time being, "list.Sort(1)" (by time) is better.

v1.02  2021-Jul-11 - by pips4life
   Found on the web a very simple 2-column sort logic now implemented as list.Sort(2), 
      which does: sort 1st priority by OpenTime, then 2nd priority by ticket#, both Ascending.  
      This should meet all FIFO criteria.
         
   Script variations:
   If one wants an immediate (no prompt) CloseAll of all open or pending orders, all Symbols, make a copy of this script
      (e.g. to "FIFO_CloseAll.mq4") and comment out the "#property script_show_inputs" so that it just runs.
   If one wants an immediate CloseAll of all open or pending orders for the current chart symbol only, make a copy of this script
      (e.g. to "FIFO_CloseAll_ThisSymbol.mq4"), then comment out the "#property script_show_inputs", AND change the
      "input MODE_SYMBOLS" default from "SYMBOLS_ALL" to "SYMBOLS_THIS"
   Other combos/variations are possible based on the choice for "input MODE_ORDER_TYPE" which has Mode_Orders = ORDERS_ALL,
      but could instead use a default of ORDERS_POSITIONS (i.e. Open orders), or ORDERS_PENDING

v1.03 2021-Jul-12 - by pips4life
   Added an Alert pop-up error message if either the "AutoTrading" icon is off, and/or the Common tab "Allow Live Trading" checkbox is off.
     Both must be on to allow the script to close trades.  Normally turning "AutoTrading" on will also enable "Allow Live Trading" 
     but both are verified since they are separate settings.
   Added an Alert pop-up warning message if no trades were closed.
   (In the code, users can change variable 'AlertVsPrint' to false if the Alert pop-ups for these new Errors/Warnings are not desired.
    Or if preferred, users can customize the code to issue an Alert or a Print for whatever error/warning conditions they want).
*/

bool AlertVsPrint=true; //true uses Alert for Errors/Warnings, false uses Print.

// nicholishen deserves the vast majority of credit for the original script.
#property copyright "Original Copyright 2017, nicholishen; 2021 v1.03 mods, pips4life"
#property link      "https://www.forexfactory.com/nicholishen" 
//#property link      "https://www.forexfactory.com/thread/716973-maximum-overkill-close-all-script"
#property version   "1.03"
#property strict
#property script_show_inputs // If this line is commented out, the script will run immediately using the defaults for all inputs
#include <Arrays\List.mqh>
#include <stdlib.mqh>

class Order : public CObject
{
public:
   int ticket;
   datetime open_time;
   Order(int ticket_number,datetime ticket_open_time)
   {
      ticket = ticket_number;
      open_time = ticket_open_time;
   }
   int Compare(const CObject *node,const int mode=0)const override
   {
      Order *other = (Order*)node;
      if(mode==2) //v1.02 Added by pips4life to do 2-column simultaneous sort, 1st by OpenTime, 2nd by ticket#, both Ascending
      {
         if(this.open_time>other.open_time)return 1;
         if(this.open_time<other.open_time)return -1;
         // At this point, open_time's are equal, so next do a 2nd priority sort by ticket#
         if(this.ticket>other.ticket)return 1;
         if(this.ticket<other.ticket)return -1;
         // If both open_time and ticket# were equal (an impossibility given unique ticket#'s), this would return 0 below, were it to occur.
      }
      else if(mode==1)
      {
         if(this.open_time>other.open_time)return 1;
         if(this.open_time<other.open_time)return -1;
      }
      else
      { 
         if(this.ticket>other.ticket)return 1;
         if(this.ticket<other.ticket)return -1;
      }
      return 0;
   }
};

enum MODE_SYMBOLS
{
   SYMBOLS_ALL, //All Symbols
   SYMBOLS_THIS //Chart Symbol Only
};

enum MODE_ORDER_TYPE
{
   ORDERS_ALL,//Close all order types
   ORDERS_POSITIONS,//Close only open positions
   ORDERS_PENDING//Close only pending orders
};

//--- input parameters
input MODE_SYMBOLS    Mode_Symbols= SYMBOLS_ALL;//Which Symbol?  SYMBOLS_ALL or SYMBOLS_THIS
input MODE_ORDER_TYPE Mode_Orders = ORDERS_ALL;//Which Order Type?  ORDERS_ALL or ORDERS_POSITIONS or ORDERS_PENDING
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
{
//---
   string msg;
   if(!IsExpertEnabled() || !IsTradeAllowed())
   {
      if(!IsExpertEnabled()) msg = "The AutoTrading icon/button is OFF, and must be Enabled to manage trades.";
      else msg = "In the Script settings -> Common tab, 'Allow Live Trading' must be checked."; 
      msg=StringConcatenate(WindowExpertName()," ",_Symbol,",",periodToString(Period())," : ERROR: ",msg);
      if(AlertVsPrint) Alert(msg);
      else Print(msg);
      return;
   }
   
   
   CList list;
   for(int i=OrdersTotal();i>=0;i--)
      if(OrderSelect(i,SELECT_BY_POS)&&(Mode_Symbols==SYMBOLS_ALL || OrderSymbol()==_Symbol))
         list.Add(new Order(OrderTicket(),OrderOpenTime()));
   
   //Old sorting:
   ////list.Sort(0); // by ticket number.  This was an experiment, but doing this 1st does not help the ultimate final order.
   //list.Sort(1); // by time
   //list.Sort(0); // then by ticket number //No, this sort breaks FIFO order, e.g. Open order #1, then #2. *Partial* close #1. The remaining 
                                            // of #1 now has a new higher ticket#, but the original OpenTime, hence for FIFO it must close first!
   
   //New list.Sort(2) true FIFO mode
   list.Sort(2); // 1st priority by open_time, 2nd priority by ticket#
   
   bool orderdeleted=false;
   for(Order *order = list.GetFirstNode();order!=NULL;order=order.Next())
   {
      if(OrderSelect(order.ticket,SELECT_BY_TICKET))
      {
         if(OrderType()>1 && (Mode_Orders==ORDERS_ALL || Mode_Orders==ORDERS_PENDING))
         {
            if(!OrderDelete(order.ticket))
               Print("OrderDeleteError: ",ErrorDescription(GetLastError()));
            else orderdeleted=true; 
            continue;
         }
         else
         if(OrderType()<2 &&(Mode_Orders==ORDERS_ALL || Mode_Orders==ORDERS_POSITIONS))
         {
            MqlTick tick;
            SymbolInfoTick(OrderSymbol(),tick);
            double price = OrderType()==OP_BUY?tick.bid:tick.ask;
            if(!OrderClose(OrderTicket(),OrderLots(),price,10))
               Print("OrderCloseError: ",ErrorDescription(GetLastError()));   
            else orderdeleted=true; 
            continue;     
         }
      }
      else
         Print("OrderSelectError: ",ErrorDescription(GetLastError()));
   }
   
   // If no trade was closed...
   if(!orderdeleted) 
   {
      msg=StringConcatenate(WindowExpertName()," ",_Symbol,",",periodToString(Period())," : WARNING: NO orders were closed.  Verify the script's Input variable choices, and whether any matching trades are open.");
      if(AlertVsPrint) Alert(msg);
      else Print(msg);
   }
}
//+------------------------------------------------------------------+

// pips4life: Maybe there's a better way to get a chart's timeframe string, but this hack works ok:
string periodToString(int period)
{
   if(period==0) period=Period();
   if(period<60) return("M"+IntegerToString(period));
   if(period<PERIOD_D1) return("H"+IntegerToString(period/60));
   if(period<PERIOD_W1) return("D"+IntegerToString(period/PERIOD_D1));
   if(period<PERIOD_MN1) return("W"+IntegerToString(period/PERIOD_W1));
   return("MN"+IntegerToString(period/PERIOD_MN1));
}